# ADMLogger by Aaron D. Marasco (Aaron@Marascos.net) - No warranties expressed nor implied.
#  Please see enclosed LICENSE for licensing terms and conditions.
#  PLEASE send me your comments!
#

# ADMLogger was a 'natural' progression of the firewall log analysis program "fireparse." I recommend that
# any programmer wanting to use ADMLogger analyze the sample plugins to see how to integrate.

##PCONFIG BEGIN NORMAL N/A
##PCONFIG ANNOUNCE #############################################################################
##PCONFIG ANNOUNCE #   This file does not use pconfig, please read the README file for setup!  #
##PCONFIG ANNOUNCE #############################################################################
##PCONFIG END NORMAL
##PCONFIG STOP

# To-do: Multiple log files...

my $_debug = 0;

=head1 NAME

ADMLogHandler.pm

=head1 USAGE

use ADMLogHandler;

=head1 DESCRIPTION

ADMLogger is an object-oriented extendable framework for log analysis tools. Many system administrators
have a handful of tools that they like to run on their logs daily/weekly/at rotation, etc.
ADMLogger attempts to bring them all together by providing a set of common tools for all perl scripts to
to maintain a consistency across all programs. It also allows the admin to only run a single program, and
for them to focus on the coding of the core of what they need to do - allowing ADMLogger to provide all
I/O support, etc.

Typical lifecycle of an ADMLogHandler object:
Contruction
Registration(s)
Execution
Retrieval
(Destruction)

=head1 BUGS

None known.

=head1 AUTHORS

Aaron D. Marasco (Aaron@Marascos.net)

=head1 LIST OF PUBLIC METHODS

 new
 register
 execute
 retrieve
 retrieve_condensed

=head1 METHODS

=over 4

=cut

package ADMLogHandler;

require File::Copy; # Is in the standard perl distro, so should be OK.
use File::Copy;
use strict;

#use Data::Dumper;

=item new

Constructor

Usage:

 new(\$pre_func, \$post_func, $log_name, $new_name);

Parameters:

 \$pre_func  - pre-processing function to execute
 \$post_func - post-processing function to execute
 $log_name   - the name of the log file
 $new_name   - the name of the processed log file

Returns:

 Reference to ADMLogHandler ready to have registrations.

=cut
sub new {
    my ($a, $b, $c, $d) = @_;
    die "Invalid pre_func in ADMLogHandler::new()!" unless ref($a);
    die "Invalid post_func in ADMLogHandler::new()!" unless ref($b);
    die "Empty log_name in ADMLogHandler::new()!" unless ($c ne '');
    die "Empty log_name in ADMLogHandler::new()!" unless ($d ne '');
    print "ADMLogHandler::new() running\n" if $_debug;
    return bless {
            'pre_func' => $a,
            'post_func' => $b,
            'log_name' => $c,
            'new_name' => $d,
            'run' => 0
    }, 'ADMLogHandler';;
}

=item register

Usage:

 register(\$regex_array, $plugin);

Parameters:

 \$regex_array - reference to an array of regular expressions
 $plugin       - name of the plugin requesting these matches

Returns:

 String - Empty if success, else error message.

=cut
sub register {
    my ($pkg, $a, $b) = @_;
    return "Too late to register! We already ran!" if ($pkg->{'run'} != 0);
    return "Invalid array given!" unless (ref($a) eq 'ARRAY');
    # I would like to verify RegEx here, saw something on CPAN for YAPE(?)
    return "Invalid plugin name given!" unless ($b ne '');
    foreach (@{$pkg->{'callers'}}) {
        print "ADMLogHandler::register checking if $b exists...\n" if $_debug;
        return "Caller $b already exists" if ($_ eq $b);
    }
    if ($_debug) {
        print "ADMLogHandler::registered $b as wanting: ";
        foreach (@$a) {
            print "$_ - ";
        };
        print "\n";
    }
    push @{$pkg->{'regexes'}}, $a; # Still a ref to an array of strings
    push @{$pkg->{'callers'}}, $b;
    return "";
}

=item execute

Usage:

 execute();

Parameters:

 none

Returns:

 String - Empty if success, else error message.

=cut
sub execute {
    # Now we need to go thru all the arrays and flatten them out.
    # If you were wondering why I had two arrays and not a hash,
    # it is because the first registered is the most important.
    my @regs = ();
    my @who = ();
    my $i = 0;
    my $pkg = shift @_;
    my $_log_name = $pkg->{'log_name'};
    my $_new_name = $pkg->{'new_name'};
    my %_results;
    my @_callers = @{$pkg->{'callers'}};
    my @_regexes = $pkg->{'regexes'};
    foreach (@_regexes) {
        foreach (@$_) { # Something in the way I push it is always a 2D array
            foreach (@$_) {
                # We should now have a reference to an array.
                print "Processing '$_': " if $_debug;
                # Now we have a string to keep
                return "Didn't find a string where I wanted one." unless (ref(\$_) eq 'SCALAR');
                push @regs, $_;
                return "I got confused somewhere and tried to tag '$_' as a regex for nobody!?" unless (defined $_callers[$i]);
                push @who, $_callers[$i];
                $_results{$_callers[$i]} = ''; # Initialize hash to a ref to an empty array
            }
            $i++; # Next caller
        }
    }
    # So now we have a flattened @regs with corresponding @who.
    if ($_debug) {
        print "ADMLogHandler::execute(): looking for the following expressions: \n";
        my $ii = 0;
        foreach (@regs) {
            print "\t$_ -- for $who[$ii]\n";
            $ii++;
        }
    }
    # Do what the user asked:
    &{$pkg->{'pre_func'}};
    # Steal the system log
    # Remember the permissions of the system log file (tnx Mika Slander)
    my $mainlogmode = (stat "$_log_name")[2];
    move ("$_log_name", "$_log_name.processing");
    # Open files
    open MAINLOG, "$_log_name.processing" or return "Could not open the renamed file! [WARNING: syslog may be STOPPED!] - $!";
    # Exclusive lock file
    flock MAINLOG, 2 or return "Could not lock the (renamed) file! [WARNING: syslog may be STOPPED!] - $!";
    # Repeat for output logs
    open NOTMYLOG, ">$_log_name" or return "Could not recreate log! [WARNING: syslog may be STOPPED!] - $!";
    flock NOTMYLOG, 2 or return "Could not lock the (new) file! [WARNING: syslog may be STOPPED!] - $!";
    open MYLOG, ">>$_new_name" or return "Could not create output log! [WARNING: syslog may be STOPPED!] - $!";
    flock MYLOG, 2 or return "Could not lock the (new) file! [WARNING: syslog may be STOPPED!] - $!";
    my $matched = 0;
    my $line = '';
    my $lastline = '';
    my $repeats = 1;
    while (<MAINLOG>) {
        $line = $_;
        $repeats = 1;
        if ($line =~ /last message repeated (\d+)/i) {
            $repeats = $1;
            $line = $lastline;
        }
        study $line;
        for (1..$repeats) {
            $i = 0;
            $matched = 0;
            foreach (@regs) {
                if ($line =~ /$_/i) {
                    # This is a keeper
                    # chomp $line;
                    # print "$repeats: Match $who[$i] : $line\n" if $_debug;
                    $_results{$who[$i]} .= $line;
                    $matched++;
                } # Match or not
                $i++;
            } # All regexes
            if ($matched != 0) {
                print MYLOG $line;
                print NOTMYLOG $line if $_debug;
            } else {
                print NOTMYLOG $line;
            } # Saved off either way
        } # Repeats
        $lastline = $line;
    }
    # OK, now we have grabbed ALL lines for everyone
    $pkg->{'run'}++;
    flock NOTMYLOG, 8; # unlocks file
    close NOTMYLOG;
    flock MYLOG, 8; # unlocks file
    close MYLOG;
    &{$pkg->{'post_func'}};
    close MAINLOG;
    chmod $mainlogmode, "$_log_name";
    unlink "$_log_name.processing";
    $pkg->{'results'} = \%_results;
    return ""; # Success!
} # execute

=item retrieve

Usage:

 retrieve($plugin);

Parameters:

 $plugin - name of the plugin requesting these matches

Returns:

 A scalar containing everything you asked for

=cut
sub retrieve {
    my ($pkg, $a) = @_;
    if (exists $pkg->{'results'}{$a}) {
        return $pkg->{'results'}{$a};
    } else {
        return '';
    }
} # retrieve

=item retrieve_condensed

Usage:

 retrieve_condensed($plugin);

Parameters:

 $plugin - name of the plugin requesting these matches

Returns:

 A reference to an array of arrays. Every element of the upper array
 is an array where [0] is the number of lines seen that day, [1] is the
 date from the condensing, and [2] is the actual line.

Example:
 Mar 30 23:03:44 hostname ftp-child[3978]: USER-WRN 'NOOP' without login from nn.nn.nn.nn
 Mar 30 23:03:44 hostname ftp-child[3978]: USER-WRN 'NOOP' without login from nn.nn.nn.nn
 Mar 30 23:04:22 hostname last message repeated 4 times

 The result would be the following (from Data::Dumper):
  $VAR1 = [
            [
              6,
              'Mar 30',
              'hostname ftp-child[3978]: USER-WRN \'NOOP\' without login from nn.nn.nn.nn'
            ]
          ];

=cut
sub retrieve_condensed {
    my ($pkg, $a) = @_;
    if (not exists $pkg->{'results'}{$a}) {
        return [0, "", ""];
    }
    my $count = 1;
    my @lines = split /\n/, $pkg->{'results'}{$a};
    my $lastline = '';
    my $lastday = '';
    my (@nums, @days, @texts);
    my $day = '';
    LINES: foreach (@lines) {
        # Now to rip out the date, time, good stuff
        /^(\w+\s+\d+)\s+\d+:\d+:\d+\s+(.*)$/;
        $day = $1;
        my $stripped = $2;
        if (($stripped eq $lastline) and ($day eq $lastday)) { # duplicate
            $count++;
            next LINES;
        }
        # Save old data
        push @nums, $count;
        $count = 1;
        push @days, $lastday;
        $lastday = $day;
        push @texts, $lastline;
        $lastline = $stripped;
    }
    # Now don't forget the one we just finished on.
    push @nums, $count;
    push @days, $lastday;
    push @texts, $lastline;
    # The first values are gonne be 1, '', '' guarenteed.
    my @result;
    for (my $i = 1; $i < (scalar @nums); $i++) {
        push (@result, [$nums[$i], $days[$i], $texts[$i]] );
    }
    return \@result;
} # retrieve_condensed

1;
