# ADMLogger by Aaron D. Marasco (Aaron@Marascos.net) - No warranties expressed nor implied.
#  Please see enclosed LICENSE for licensing terms and conditions.
#  PLEASE send me your comments!
#

# ADMLogger was a 'natural' progression of the firewall log analysis program "fireparse." I recommend that
# any programmer wanting to use ADMLogger analyze the sample plugins to see how to integrate.

##PCONFIG BEGIN NORMAL N/A
##PCONFIG ANNOUNCE #############################################################################
##PCONFIG ANNOUNCE #   This file does not use pconfig, please read the README file for setup!  #
##PCONFIG ANNOUNCE #############################################################################
##PCONFIG END NORMAL
##PCONFIG STOP

# To-do: Attachments!

=head1 NAME

ADMMailHandler.pm

=head1 USAGE

use ADMMailHandler;

=head1 DESCRIPTION

ADMLogger is an object-oriented extendable framework for log analysis tools. Many system administrators
have a handful of tools that they like to run on their logs daily/weekly/at rotation, etc.
ADMLogger attempts to bring them all together by providing a set of common tools for all perl scripts to
to maintain a consistency across all programs. It also allows the admin to only run a single program, and
for them to focus on the coding of the core of what they need to do - allowing ADMLogger to provide all
I/O support, etc.

Typical lifecycle of an ADMMailHandler object:
Contruction
Subject
Building
Sending
(Destruction)

=head1 BUGS

None known.

=head1 AUTHORS

Aaron D. Marasco (Aaron@Marascos.net)

=head1 LIST OF PUBLIC METHODS

 new
 subject
 append
 checkflag
 clearflag
 isHTML
 setHTML
 isSendable
 sendit

=head1 METHODS

=over 4

=cut

package ADMMailHandler;
use strict;
# use Data::Dumper;

my $_debug = 0;

=item new

Constructor

Usage:

 new($to, $html, $subject);

Parameters:

 $to      - e-mail address message will be sent to
 $html    - flag to send HTML MIME Type or not
 $subject - subject of email (optional)
            can be changed - see subject()

Returns:

 Reference to a ready ADMMailHandler.

=cut
sub new {
    my ($a, $b, $c) = @_;
    die "Invalid email address in ADMMailHandler::new()!" unless ($a =~ /^[\w.]+\@(\w+\.)+\w+$/);
    die "Invalid html flag ADMMailHandler::new()!" unless (($b == 0) or ($b == 1));
    $c = '(No subject specified)' unless defined $c;
    return bless {
            'to' => "$a",
            'html' => "$b",
            'subject' => "$c",
            'run' => 0,
            'flag' => 0
    }, 'ADMMailHandler';
}

=item subject

Changes the subject of the outgoing message

Usage:

 subject($subject);

Parameters:

 $subject - subject of email

Returns:

 string - Empty if success, else error message.

=cut
sub subject {
    my ($pkg, $a) = @_;
    return "Too late to change the subject! We already sent!" if ($pkg->{'run'} != 0);
    return "No newlines please!" if ($a =~/\n/);
    $pkg->{'subject'} = $a;
    return '';
}

=item append

Usage:

 append($string);

Parameters:

 $string            - a single string to add

Returns:

 string - Empty if success, else error message (none defined).

=cut

sub append {
    my ($pkg, $a) = @_;
    push @{$pkg->{'body'}}, $a;
    $pkg->{'flag'} = 1;
    return '';
}

=item checkflag

Checks if message has been appended to, either ever or since last clearflag call

Usage:

 checkflag();

Parameters:

 none

Returns:

 1 if data has changed, 0 if not.

=cut
sub checkflag {
    my ($pkg) = @_;
    return $pkg->{'flag'};
} # checkflag

=item clearflag

Clears 'message modified' flag. Useful to refrain from sending useless messages.

Usage:

 clearflag();

Parameters:

 none

Returns:

 1 always.

=cut
sub clearflag {
    my ($pkg) = @_;
    $pkg->{'flag'} = 0;
    return 1;
} # clearflag

=item isHTML

Checks if message will be HTML formatted

Usage:

 isHTML();

Parameters:

 none

Returns:

 1 if HTML enabled, 0 if not.

=cut
sub isHTML {
    my ($pkg) = @_;
    return $pkg->{'html'};
} # isHTML

=item setHTML

Can change default HTML options

Usage:

 setHTML($flag);

Parameters:

 $flag - 1 if HTML enabled, 0 if not.

Returns:

 0 if $flag is invalid, otherwise 1.

=cut
sub setHTML {
    my ($pkg, $f) = @_;
    return 0 unless ($f =~ /^[01]$/);
    $pkg->{'html'} = $f;    
    return 1;    
} # setHTML


=item isSendable

Checks if message can be sent in current state

Usage:

 isSendable();

Parameters:

 none

Returns:

 1 if sendit should succeed, 0 if not.

=cut
sub isSendable {
    my ($pkg) = @_;
    return 0 if ($pkg->{'run'} != 0);
    return 0 unless defined $pkg->{'body'};
    return 1;
} # isSendable

=item sendit

Usage:

 sendit($mailer);

Parameters:

 $mailer - mail executable to use (optional)
           defaults to "/usr/sbin/sendmail -t"

Returns:

 string - Empty if success, else error message.

=cut
sub sendit {
    my $mail = "/usr/sbin/sendmail -t";
    my ($pkg, $a) = @_;
    return "Refuse to spam! We already sent!" if ($pkg->{'run'} != 0);
    return "Empty messages are useless!" unless defined $pkg->{'body'};
    $mail = $a if (defined $a);
    my $_to = $pkg->{'to'};
    my $_subject = $pkg->{'subject'};
    my $_html = $pkg->{'html'};
    my @_body = @{$pkg->{'body'}};
    open (NewMsg, "|$mail") or return "Cannot open mail handle!";
    print NewMsg <<HEADER;
To: $_to
Subject: $_subject
MIME-Version: 1.0
HEADER
    print NewMsg ($_html == 1)?"Content-Type: text/html; charset=\"us-ascii\"\n\n":"\n";
    if ($_html == 1) {
        print NewMsg <<HTMLHEADER;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd">
<HTML><HEAD><META CREATOR="Aaron D. Marasco"><TITLE>$_subject</TITLE></HEAD><BODY>
HTMLHEADER
    } else {
        print NewMsg "\n\n";
    }
    foreach (@_body) {
        print NewMsg $_;
    }
    print NewMsg "<BR></BODY></HTML>" if ($_html == 1);
    # Send the e-mail
    close (NewMsg);
    $pkg->{'run'}++;
    return ""; # Success!
} # sendit

1;
