# ADMLogger by Aaron D. Marasco (Aaron@Marascos.net) - No warranties expressed nor implied.
#  Please see enclosed LICENSE for licensing terms and conditions.
#  PLEASE send me your comments!
#

# ADMLogger was a 'natural' progression of the firewall log analysis program "fireparse." I recommend that
# any programmer wanting to use ADMLogger analyze the sample plugins to see how to integrate.

##PCONFIG BEGIN NORMAL N/A
##PCONFIG ANNOUNCE #############################################################################
##PCONFIG ANNOUNCE #   This file does not use pconfig, please read the README file for setup!  #
##PCONFIG ANNOUNCE #############################################################################
##PCONFIG END NORMAL
##PCONFIG STOP

=head1 NAME

ADMNetHandler.pm

=head1 USAGE

use ADMNetHandler;

=head1 DESCRIPTION

ADMLogger is an object-oriented extendable framework for log analysis tools. Many system administrators
have a handful of tools that they like to run on their logs daily/weekly/at rotation, etc.
ADMLogger attempts to bring them all together by providing a set of common tools for all perl scripts to
to maintain a consistency across all programs. It also allows the admin to only run a single program, and
for them to focus on the coding of the core of what they need to do - allowing ADMLogger to provide all
I/O support, etc.

=head1 BUGS

None known.

=head1 AUTHORS

Aaron D. Marasco (Aaron@Marascos.net)

=head1 LIST OF PUBLIC METHODS

 new
 DNSLookup
 ServiceName

=head1 METHODS

=over 4

=cut

package ADMNetHandler;
use strict;

require Socket; # Socket is in the standard perl distro, so should be OK.
use Socket;

# use Data::Dumper;
my $_debug = 0;

=item new

Constructor

Usage:

 new();

Parameters:

 none

Returns:

 Reference to a ready ADMNetHandler.

=cut
sub new {
    my %c;
    my %services;
    my @service; # Must declare outside of while loops
    # Parse the services file. This will give us a hash with the name of all the port services
    print STDOUT "Examining services... " if $_debug;
    foreach my $fname ('/etc/services', '/etc/services.fake', '/var/adm/logger/services.fake') {
        if (open SERVICEFILE, "< $fname") {
            while (<SERVICEFILE>) {
                chomp; # remove newline
                @service = split; # parse and remove whitespace
                if (scalar @service >= 2) {
                    # Only relevant info for us are first two fields in services
                    if (($service[0] !~ "#") and ((length $service[0]) > 2) and ($service[1] =~ "/") ) {
                        # Not a comment or a blank line
                        $services{$service[1]} = $service[0]; # name now stored in hash
                    } # not a comment
                } # not blank line
            } # end of file
            close SERVICEFILE;
        } # File exists or not
    }
    return bless {
        'cache' => \%c,
        'services' => \%services
    }, 'ADMNetHandler';
} # new 

=item DNSLookup

Looks up real hostname from IP (with caching)

Usage:

 DNSLookup($IPAddr);

Parameters:

 $IPAddr - Address to look up (string form)

Returns:

 String - FQDN of IP or numeric IP if not found.

=cut
sub DNSLookup {
    my ($pkg, $addy) = @_;
    my $exthost = "";
    # Do a reverse DNS lookup
    if ( defined $pkg->{'cache'}{$addy} ) { # Cache hit
      $exthost = $pkg->{'cache'}{$addy};
      print STDERR "Cache hit for ".$addy." ==> ".$exthost."\n" if $_debug;
    } elsif ($addy =~ /\.255$/) { # Skip broadcasts
      $exthost = $addy;
    } else { # Cache miss
      my $tempo = gethostbyaddr (inet_aton($addy), Socket::AF_INET);
      if (defined ($tempo)) { # valid, set and save to cache
        $exthost = $tempo;
        $pkg->{'cache'}{$addy} = $exthost;
      } else { # Not valid, but don't look up again either.
        $exthost = $addy;
	$pkg->{'cache'}{$addy} = $addy;
      } # gethostbyaddr result
    } # Cache
    return $exthost;
}

=item ServiceName

Looks up human readable name for a port/protocol pair.

Usage:

 ServiceName($Pair);

Parameters:

 $Pair - "Port/Protocol" as a string

Returns:

 String - Service name (if found in /etc/services or /etc/services.fake), else same as input.

=cut
sub ServiceName {
    my ($pkg, $pair) = @_;
    my %services = %{$pkg->{'services'}};
    if (defined $services{$pair}) {
      print STDERR "Cache hit for $pair ==> $services{$pair}\n" if $_debug;
      $pair = $services{$pair};
    } # Now $pair should be either text or port/proto - no matter
    return $pair;
}

1;
